import django_filters

from . import models
from utilities.filters import TagFilter

from dcim.models import Device
from tenancy.models import Tenant

class LifecycleActionTypesFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(
        field_name='name',
        lookup_expr='icontains',
        label='Name',
    )

    class Meta:
        model = models.LifecycleActionType
        fields = ['id', 'name']

class LifecyclesFilter(django_filters.FilterSet):
    date = django_filters.DateFilter(
        field_name='date',
        lookup_expr='icontains',
        label='Date',
    )

    device = django_filters.ModelMultipleChoiceFilter(
        field_name='device_id',
        queryset=Device.objects.all(),
        to_field_name='id',
        label='Device ID',
    )

    action_type = django_filters.ModelMultipleChoiceFilter(
        field_name='action_type_id',
        queryset=models.LifecycleActionType.objects.all(),
        to_field_name='id',
        label='Action Type ID',
    )

    user = django_filters.ModelMultipleChoiceFilter(
        field_name='user_id',
        queryset=Tenant.objects.all(),
        to_field_name='id',
        label='User ID',
    )

    tag = TagFilter()

    class Meta:
        model = models.Lifecycle
        fields = ['id', 'date']



# Generated by Django 3.0.8 on 2020-07-17 15:09

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('lifecycles', '0002_auto_20200717_1506'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lifecycle',
            name='date',
            field=models.DateField(default=django.utils.timezone.now),
        ),
    ]



# Generated by Django 3.0.8 on 2020-07-17 15:06

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('lifecycles', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lifecycle',
            name='date',
            field=models.DateField(default=datetime.datetime(2020, 7, 17, 15, 6, 12, 223023)),
        ),
        migrations.AlterField(
            model_name='lifecycleactiontype',
            name='name',
            field=models.DateField(),
        ),
    ]



from extras.plugins import PluginTemplateExtension
from .models import Lifecycle

class DeviceLifecycles(PluginTemplateExtension):
    model = 'dcim.device'

    def right_page(self):
        return self.render('lifecycles/inc/device_lifecycles.html', extra_context={
	'lifecycles':Lifecycle.objects.filter(device_id=self.context['object'].id).order_by('date').all(),
	'device': self.context['object'].id
        })

template_extensions = [DeviceLifecycles]



# Generated by Django 3.0.8 on 2020-07-22 12:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('lifecycles', '0005_auto_20200722_1212'),
    ]

    operations = [
        migrations.AddField(
            model_name='lifecycle',
            name='created',
            field=models.DateField(auto_now_add=True, null=True),
        ),
        migrations.AddField(
            model_name='lifecycle',
            name='last_updated',
            field=models.DateTimeField(auto_now=True, null=True),
        ),
    ]



# Generated by Django 3.0.8 on 2020-07-16 11:13

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('dcim', '0106_role_default_color'),
        ('tenancy', '0009_standardize_description'),
    ]

    operations = [
        migrations.CreateModel(
            name='LifecycleActionType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('name', models.DateField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Lifecycle',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('date', models.DateField(blank=True, null=True)),
                ('action_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='lifecycles.LifecycleActionType')),
                ('device', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dcim.Device')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='tenancy.Tenant')),
            ],
        ),
    ]



from extras.plugins import PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices

menu_items = (
    PluginMenuItem(
        link='plugins:lifecycles:lifecycles',
        link_text='Lifecycles',
		permissions=['lifecycles.view_lifecycle'],
        buttons=(
            PluginMenuButton('plugins:lifecycles:lifecycle_add', 'Add lifecycle', 'fa fa-plus', ButtonColorChoices.GREEN,
							permissions=['lifecycles.add_lifecycle']),
        )
    ),
    PluginMenuItem(
        link='plugins:lifecycles:lifecycle_action_types',
        link_text='Lifecycle Action Types',
		permissions=['lifecycles.view_lifecycleactiontype'],
        buttons=(
            PluginMenuButton('plugins:lifecycles:lifecycle_action_type_add', 'Add lifecycle action type', 'fa fa-plus', ButtonColorChoices.GREEN,
							permissions=['lifecycles.add_lifecycleactiontype']),
        )
    )
)



from django.urls import path
from . import views, models

from extras.views import ObjectChangeLogView

urlpatterns = [
    path('lifecycles/<int:pk>/', views.LifecycleView.as_view(), name='lifecycle'),
    path('lifecycles/create/', views.LifecyclesCreateView.as_view(), name='lifecycle_add'),
    path('lifecycles/', views.LifecyclesView.as_view(), name='lifecycles'),
    path('lifecycles/delete/', views.LifecyclesBulkDeleteView.as_view(), name='lifecycle_bulk_delete'),
    path('lifecycles/<int:pk>/delete/', views.LifecyclesDeleteView.as_view(), name='lifecycle_delete'),
    path('lifecycles/<int:pk>/edit/', views.LifecyclesEditView.as_view(), name='lifecycle_edit'),
    path('lifecycles/<int:pk>/changelog/',  ObjectChangeLogView.as_view(), name='lifecycle_changelog', kwargs={'model':models.Lifecycle}),

    path('lifecycle_action_types/create/', views.LifecycleActionTypesCreateView.as_view(), name='lifecycle_action_type_add'),
    path('lifecycle_action_types/', views.LifecycleActionTypesView.as_view(), name='lifecycle_action_types'),
    path('lifecycle_action_types/delete/', views.LifecycleActionTypesBulkDeleteView.as_view(), name='lifecycle_action_type_bulk_delete'),
    path('lifecycle_action_types/<int:pk>/delete/', views.LifecycleActionTypesDeleteView.as_view(), name='lifecycle_action_type_delete'),
    path('lifecycle_action_types/<int:pk>/edit/', views.LifecycleActionTypesEditView.as_view(), name='lifecycle_action_type_edit'),
]



from django.db import models
from dcim.models import Device
from tenancy.models import Tenant
from taggit.managers import TaggableManager
from extras.models import CustomFieldModel, ObjectChange, TaggedItem
from utilities.models import ChangeLoggedModel

from django.utils import timezone
from django.contrib.contenttypes.fields import GenericRelation
from django.urls import reverse


class LifecycleActionType(models.Model):
	name = models.CharField(max_length=50)

	def __str__(self):
          return getattr(self, 'name')

	def get_absolute_url(self):
           return reverse('plugins:lifecycles:lifecycle_action_types')

class Lifecycle(ChangeLoggedModel):
	date = models.DateField(default=timezone.now)
	user = models.ForeignKey(Tenant, on_delete=models.PROTECT, blank=True, null=True)
	device = models.ForeignKey(Device, on_delete=models.PROTECT)
	action_type = models.ForeignKey(LifecycleActionType, on_delete=models.PROTECT)

	def __str__(self):
          return str(getattr(self, 'date'))+': '+ getattr(self, 'action_type').name +' - '+ getattr(self, 'device').name

	comments = models.TextField(
           blank=True
        )

	tags = TaggableManager(through=TaggedItem)

	class Meta:
	   ordering = ['id']

	def get_absolute_url(self):
           return reverse('plugins:lifecycles:lifecycle', args=[self.id])



# Generated by Django 3.0.8 on 2020-07-17 16:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('lifecycles', '0003_auto_20200717_1509'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lifecycleactiontype',
            name='name',
            field=models.CharField(max_length=50),
        ),
    ]



from django import forms

from utilities.forms import APISelect, BootstrapMixin, DynamicModelMultipleChoiceField, APISelectMultiple
from .models import Lifecycle, LifecycleActionType
from dcim.models import Device
from tenancy.models import Tenant

from extras.forms import TagField

class LifecyclesCreateForm(BootstrapMixin, forms.ModelForm):
    tags = TagField(required=False)

    class Meta:
        model = Lifecycle
        fields = [
            'id', 'date', 'user', 'device', 'action_type', 'comments', 'tags'
        ]
        widgets = {
            'device': APISelect(api_url='/api/dcim/devices/'),
            'user': APISelect(api_url='/api/tenancy/tenants/'),
            'action_type': APISelect(api_url='/api/plugins/lifecycles/lifecycle_action_types')
        }

class LifecycleActionTypesCreateForm(BootstrapMixin, forms.ModelForm):
    class Meta:
        model = LifecycleActionType
        fields = [
            'name'
        ]
        widgets = { }

class LifecycleActionTypesFilterForm(BootstrapMixin, forms.Form):
    model = LifecycleActionType
    name = forms.CharField(
        required=False,
        label='Search'
    )

class LifecyclesFilterForm(BootstrapMixin, forms.Form):
    model = Lifecycle

    device = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        to_field_name='id',
        required=False,
        widget=APISelectMultiple(
            value_field="id",
        )
    )

    action_type = DynamicModelMultipleChoiceField(
        queryset=LifecycleActionType.objects.all(),
        to_field_name='id',
        required=False,
        widget=APISelectMultiple(
            api_url='/api/plugins/lifecycles/lifecycle_action_types',
            value_field="id",
        )
    )

    user = DynamicModelMultipleChoiceField(
        queryset=Tenant.objects.all(),
        to_field_name='id',
        required=False,
        widget=APISelectMultiple(
            value_field="id",
        )
    )


    date = forms.DateField(
        required=False,
        label='Date'
    )




# Generated by Django 3.0.8 on 2020-07-22 16:13

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tenancy', '0009_standardize_description'),
        ('dcim', '0106_role_default_color'),
        ('lifecycles', '0006_auto_20200722_1253'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='lifecycle',
            options={'ordering': ['id']},
        ),
        migrations.AlterField(
            model_name='lifecycle',
            name='action_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='lifecycles.LifecycleActionType'),
        ),
        migrations.AlterField(
            model_name='lifecycle',
            name='device',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='dcim.Device'),
        ),
        migrations.AlterField(
            model_name='lifecycle',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='tenancy.Tenant'),
        ),
    ]



from django.shortcuts import render, redirect, get_object_or_404
from django.views.generic import View
from django.contrib import messages
from django.contrib.auth.mixins import PermissionRequiredMixin

from utilities.views import BulkDeleteView, ObjectDeleteView, ObjectEditView, ObjectListView
from dcim.models import Device

from .models import Lifecycle, LifecycleActionType
from . import tables, filters, forms

class LifecycleView(PermissionRequiredMixin, View):
    permission_required = 'lifecycles.view_lifecycle'

    def get(self, request, pk):
        lifecycle = get_object_or_404(Lifecycle, id=pk)
        return render(request, 'lifecycles/lifecycle.html', {
            'lifecycle': lifecycle
        })

class LifecyclesView(PermissionRequiredMixin, ObjectListView):
  permission_required = 'lifecycles.view_lifecycle'
  queryset = Lifecycle.objects.prefetch_related('device')
  filterset = filters.LifecyclesFilter
  filterset_form = forms.LifecyclesFilterForm
  table = tables.LifecyclesTable
  template_name = 'lifecycles/lifecycles_list.html'

class LifecyclesCreateView(PermissionRequiredMixin, ObjectEditView):
    permission_required = 'lifecycles.add_lifecycle'
    model = Lifecycle
    model_form = forms.LifecyclesCreateForm
    template_name = 'lifecycles/lifecycles_edit.html'
    default_return_url = 'plugins:lifecycles:lifecycles'

    def alter_obj(self, obj, request, url_args, url_kwargs):
        if 'device' in request.GET:
            obj.device = get_object_or_404(Device, pk=request.GET['device'])
        return obj

    def get_return_url(self, request, service):
        if 'device' in request.GET:
            return '/dcim/devices/'+str(request.GET['device'])+'/'
        else:
            return '/plugins/lifecycles/lifecycles/'

class LifecyclesEditView(LifecyclesCreateView):
  permission_required = 'lifecycles.change_lifecycle'


class LifecyclesBulkDeleteView(PermissionRequiredMixin, BulkDeleteView):
    permission_required = 'lifecyclese.delete_lifecycle'
    queryset = Lifecycle.objects.all()
    filter = filters.LifecyclesFilter
    table = tables.LifecyclesTable
    default_return_url = 'plugins:lifecycles:lifecycles'


class LifecyclesDeleteView(PermissionRequiredMixin, ObjectDeleteView):
    permission_required = 'lifecycles.delete_lifecycle'
    model = Lifecycle
    default_return_url = 'plugins:lifecycles:lifecycles'

class LifecycleActionTypesView(PermissionRequiredMixin, ObjectListView):
  permission_required = 'lifecycles.view_lifecycleactiontype'
  queryset = LifecycleActionType.objects.all()
  filterset = filters.LifecycleActionTypesFilter
  filterset_form = forms.LifecycleActionTypesFilterForm
  table = tables.LifecycleActionTypesTable
  template_name = 'lifecycles/lifecycle_action_types_list.html'

class LifecycleActionTypesCreateView(PermissionRequiredMixin, ObjectEditView):
    permission_required = 'lifecycles.add_lifecycleactiontype'
    model = LifecycleActionType
    model_form = forms.LifecycleActionTypesCreateForm
    template_name = 'lifecycles/lifecycle_action_types_edit.html'
    default_return_url = 'plugins:lifecycles:lifecycle_action_types'

class LifecycleActionTypesEditView(LifecycleActionTypesCreateView):
  permission_required = 'lifecycles.change_lifecycleactiontype'

class LifecycleActionTypesBulkDeleteView(PermissionRequiredMixin, BulkDeleteView):
    permission_required = 'lifecyclese.delete_lifecycleactiontype'
    queryset = LifecycleActionType.objects.all()
    filter = filters.LifecycleActionTypesFilter
    table = tables.LifecycleActionTypesTable
    default_return_url = 'plugins:lifecycles::lifecycle_action_types'


class LifecycleActionTypesDeleteView(PermissionRequiredMixin, ObjectDeleteView):
    permission_required = 'lifecycles.delete_lifecycleactiontype'
    model = LifecycleActionType
    default_return_url = 'plugins:lifecycles::lifecycle_action_types'



from extras.plugins import PluginConfig

class LifecyclesConfig(PluginConfig):
    name = 'lifecycles'
    verbose_name = 'lyfecycles'
    description = 'Lifecycles for devices'
    version = '1.0.4'
    author = 'DCIM'
    author_email = 'dcim@selectel.ru'
    base_url = 'lifecycles'
    required_settings = []
    default_settings = {
        'loud': False
    }

config = LifecyclesConfig



from rest_framework import routers
from . import views

class LifecyclesRootView(routers.APIRootView):
    def get_view_name(self):
        return 'Lifecycles'

router = routers.DefaultRouter()
router.APIRootView = LifecyclesRootView
router.register('lifecycles', views.LifecycleViewSet)
router.register('lifecycle_action_types', views.LifecycleActionTypeViewSet)
urlpatterns = router.urls



# Generated by Django 3.0.8 on 2020-07-22 12:12

from django.db import migrations, models
import taggit.managers


class Migration(migrations.Migration):

    dependencies = [
        ('extras', '0042_customfield_manager'),
        ('lifecycles', '0004_auto_20200717_1625'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='lifecycle',
            options={'ordering': ['device', 'date', 'action_type']},
        ),
        migrations.AddField(
            model_name='lifecycle',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='lifecycle',
            name='tags',
            field=taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag'),
        ),
    ]



from rest_framework import serializers
from dcim.api.nested_serializers import NestedDeviceSerializer
from tenancy.api.nested_serializers import NestedTenantSerializer

from ..models import Lifecycle, LifecycleActionType

class LifecycleActionTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = LifecycleActionType
        fields = ['id', 'name']

class LifecycleSerializer(serializers.ModelSerializer):
    action_type = LifecycleActionTypeSerializer()
    device = NestedDeviceSerializer()
    user = NestedTenantSerializer()

    class Meta:
        model = Lifecycle
        fields = ['id', 'date', 'device', 'user', 'action_type']



from setuptools import find_packages, setup

setup(
    name='netbox-plugin-lifecycles',
    version='1.0.4',
    description='Lifecycles for devices',
    url='https://git.selectel.org/ogorov/netbox-lifecycles.git',
    author='DCIM',
    license='Apache 2.0',
    install_requires=[],
    packages=find_packages(),
    include_package_data=True,
)



from rest_framework.viewsets import ModelViewSet
from lifecycles.models import LifecycleActionType, Lifecycle
from .serializers import LifecycleActionTypeSerializer, LifecycleSerializer

class LifecycleActionTypeViewSet(ModelViewSet):
    queryset = LifecycleActionType.objects.all()
    serializer_class = LifecycleActionTypeSerializer

class LifecycleViewSet(ModelViewSet):
    queryset = Lifecycle.objects.all()
    serializer_class = LifecycleSerializer



from utilities.tables import BaseTable, ToggleColumn
import django_tables2 as tables
from django_tables2.utils import Accessor

from . import models

class LifecycleActionTypesTable(BaseTable):
    pk = ToggleColumn()

    actions = tables.TemplateColumn(
        template_name='lifecycles/lifecycle_action_types_buttons.html',
        attrs={'td': {'class': 'text-right noprint'}},
        verbose_name=''
    )

    class Meta(BaseTable.Meta):
        model = models.LifecycleActionType
        fields = (
            'pk', 'name', 'actions'
        )

class LifecyclesTable(BaseTable):
    pk = ToggleColumn()
    device = tables.LinkColumn()

    id = tables.LinkColumn(
        viewname='plugins:lifecycles:lifecycle',
        args=[Accessor('pk')],
        verbose_name='ID'
    )

    actions = tables.TemplateColumn(
        template_name='lifecycles/lifecycles_buttons.html',
        attrs={'td': {'class': 'text-right noprint'}},
        verbose_name=''
    )

    class Meta(BaseTable.Meta):
        model = models.Lifecycle
        fields = (
            'pk', 'id', 'date', 'action_type', 'device', 'actions'
        )
